@@ -831,3 +831,67 @@ class CppcheckDriver(QualityDriver):
         Returns: boolean True if installed
         """
         return run_command_for_code(self.command_to_check_install) == 0
+
+
+class ClangFormatDriver(QualityDriver):
+    """
+    Driver for clang-format
+    """
+
+    def __init__(self):
+        """
+        args:
+            expression: regex used to parse report
+        See super for other args
+        """
+        super().__init__(
+            "clang",
+            ["c", "cpp", "h", "hpp"],
+            ["clang-format", "--dry-run"],  # Use dry-run option to not modify files
+            output_stderr=True,
+        )
+
+        # Errors look like:
+        # src/foo.c:8:1: warning: code should be clang-formatted [-Wclang-format-violations]
+        # int foo;
+        #   ^
+        # Match for everything, including ":" in the file name (first capturing
+        # group), in case there are pathological path names with ":"
+        self.clang_expression = re.compile(
+            r"^(.*?):(\d+):\d+: ([^[]* \[[^]]*\])\n(.+)\n(.+)$",
+            re.MULTILINE,
+        )
+        self.command_to_check_install = ["clang-format", "--version"]
+
+    def parse_reports(self, reports):
+        """
+        Args:
+            reports: list[str] - output from the report
+        Return:
+            A dict[Str:Violation]
+            Violation is a simple named tuple Defined above
+        """
+        violations_dict = defaultdict(list)
+        for report in reports:
+
+            matches = list(re.finditer(self.clang_expression, report))
+            for match in matches:
+                if match is not None:
+                    (
+                        clang_src_path,
+                        line_number,
+                        message,
+                        code_extract,
+                        cursor_error,
+                    ) = match.groups()
+                    full_message = f"{message}\n{code_extract}\n{cursor_error}"
+                    violation = Violation(int(line_number), full_message)
+                    violations_dict[clang_src_path].append(violation)
+        return violations_dict
+
+    def installed(self):
+        """
+        Method checks if the provided tool is installed.
+        Returns: boolean True if installed
+        """
+        return run_command_for_code(self.command_to_check_install) == 0
